@@ -157,6 +157,9 @@ Using `cut "0"` is effectively _no_ cutgroup.
 (lclaves, lclaves_p)             = pF "lclaves" (Just 0)
 (lclhat, lclhat_p)               = pF "lclhat" (Just 0)
 (lcrash, lcrash_p)               = pF "lcrash" (Just 0)
+(leslie, leslie_p)               = pF "leslie" (Just 0)
+(lrate, lrate_p)                 = pF "lrate" (Just 0)
+(lsize, lsize_p)                 = pF "lsize" (Just 0) 
 (lfo, lfo_p)                     = pF "lfo" (Just 0)
 (lfocutoffint, lfocutoffint_p)   = pF "lfocutoffint" (Just 0)
 (lfodelay, lfodelay_p)           = pF "lfodelay" (Just 0)
