@@ -13,9 +13,10 @@ import Data.Ratio
 import Data.Typeable
 import Sound.Tidal.Pattern
 import qualified Sound.Tidal.Parse as P
-import Sound.Tidal.Tempo (Tempo, logicalTime, clocked,clockedTick,cps)
+import Sound.Tidal.Tempo (Tempo, logicalTime, clockedTick,cps)
 import Sound.Tidal.Utils
 import qualified Sound.Tidal.Time as T
+import System.IO (stderr, hPutStrLn)
 
 import qualified Data.Map.Strict as Map
 
@@ -162,7 +163,7 @@ onTick backend shape patternM change ticks
            messages = mapMaybe
                       (toMessage backend shape change ticks)
                       (seqToRelOnsetDeltas (a, b) p)
-       E.catch (sequence_ messages) (\msg -> putStrLn $ "oops " ++ show (msg :: E.SomeException))
+       E.catch (sequence_ messages) (\msg -> hPutStrLn stderr $ "failed " ++ show (msg :: E.SomeException))
        flush backend shape change ticks
        return ()
 
@@ -326,4 +327,3 @@ coerce par@(F _ _) p = (Map.update f par) <$> p
   where f (VS s) = Just (VF $ read s)
         f (VI i) = Just (VF $ fromIntegral i)
         f (VF f) = Just (VF f)
-
